
package jsparky;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import jsparky.operations.*;

public class App {

    public OperationManager manager;

    public RSTTransformer transformer;

    public Realizer realizer;

    public App(String dss_dir) {
        realizer = new Realizer();
        realizer.loadDSSFiles(dss_dir);
        transformer = new RSTTransformer();
        manager = new OperationManager();
        manager.setRealizer(realizer);
        // Assign probabilities to operations
        setupOperationManager();
    }

    public SentencePlan process(String speech_plan_path) {
        SpeechPlan splan = new SpeechPlan(speech_plan_path);
        splan.bindDSS(realizer);
        try {
            RSTPlan rstplan = transformer.transform(splan.getRelations());
            System.out.println(Utils.serialize(rstplan.getDocumentElement()));
            SentencePlan sentplan = manager.aggregate(rstplan);
            showResult(sentplan);
            return sentplan;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public void showResult(SentencePlan splan) {
        AggregatedElement a = (AggregatedElement) splan.getDocumentElement();
        String text = manager.getText(a);
        String newtext = Utils.process_rightmost_comma(text);
        System.out.println("\nfinal : " + newtext + "\n");
    }

    /** setup for sparky operation * */
    public void setupOperationManager() {
        OperationGroup group1 = new OperationGroup();
        OperationGroup group2 = new OperationGroup();
        OperationGroup group3 = new OperationGroup();
        OperationGroup group4 = new OperationGroup();

        group1.addOperation(Operation.merge_op);
        group1.addOperation(Operation.relative_clause_op);
        group1.addOperation(Operation.with_ns_op);
        group1.addOperation(Operation.with_sn_op);

        group2.addOperation(Operation.cw_conjunction_op);
        group2.addOperation(Operation.cw_because_ns_op);
        group2.addOperation(Operation.cw_because_sn_op);
        group2.addOperation(Operation.cw_since_ns_op);
        group2.addOperation(Operation.cw_since_sn_op);
        group2.addOperation(Operation.cw_while_op);
        group2.addOperation(Operation.cw_but_op);

        group3
                .addOperation(Operation.period_cw_on_the_other_hand_full_scope_op);
        group3
                .addOperation(Operation.period_cw_on_the_other_hand_subject_scope_op);
        group3.addOperation(Operation.period_cw_however_full_scope_op);
        group3.addOperation(Operation.period_cw_however_subject_scope_op);

        group4.addOperation(Operation.period_op);

        manager.register(group1, 80);
        manager.register(group2, 10);
        manager.register(group3, 9);
        manager.register(group4, 1);
    }

    public String getFinalText(SentencePlan splan) {
        AggregatedElement a = (AggregatedElement) splan.getDocumentElement();
        String text = manager.getText(a);
        String newtext = Utils.process_rightmost_comma(text);
        return newtext;
    }

    // generate different num sentences of tplan
    public List generateAlt(String tplan, int num) {
        ArrayList finalTexts = new ArrayList();
        ArrayList splanList = new ArrayList();
        int max = 1000;
        for (int i = 0; i < max; i++) {
            
            // produce random sentence plan for this text plan
            SentencePlan splan = process(tplan);
            if (splan != null) {
                String finalText = getFinalText(splan);
                if (!finalTexts.contains(finalText)) {
                    finalTexts.add(finalText);
                    splanList.add(splan);

                }
            }
            if (finalTexts.size() >= num) {
                return finalTexts;
            }
        }
        System.out.println("Warning : " + num + " not reached within " + max
                + " attempts");
        return finalTexts;
    }
    
    

    
    

    /** get all recommend tplans * */
    public List getTplans(String type) {
        ArrayList lis = new ArrayList();
        String categories[] = { "cheap", "french", "eastvillagejapanese",
                "italianwestvillage", "midtownwest", "upperwestsideasian" };
        for (int i = 0; i < categories.length; i++) {
            List l = getTplans(type, categories[i]);
            lis.addAll(l);
        }
        return lis;
    }

    public List getTplans(String type, String category) {
        ArrayList lis = new ArrayList();
        Vector tplans = Utils.getFiles("/home/francois/SPaRKy/infiles/tplans/"
                + category, "xml");
        for (int i = 0; i < tplans.size(); i++) {
            File tplan = (File) tplans.elementAt(i);
            if (tplan.getPath().indexOf(type) != -1) {
                lis.add(tplan.getPath());
            }
        }
        return lis;
    }

    public void testByPlan(String tplan) {
        manager.setDebug(true);
        process(tplan);
    }

    public void testByCategory(String category) {
        manager.setDebug(true);
        Vector tplans = Utils.getFiles("/home/francois/SPaRKy/infiles/tplans/"
                + category, "xml");
        for (int i = 0; i < tplans.size(); i++) {
            File tplan = (File) tplans.elementAt(i);
            process(tplan.getPath());
        }
    }

    public void testAllCategory() {
        String categories[] = { "cheap", "french", "eastvillagejapanese",
                "italianwestvillage", "midtownwest", "upperwestsideasian" };
        for (int i = 0; i < categories.length; i++) {
            testByCategory(categories[i]);
        }
    }

    public void testSimple() {
        testByPlan("/home/francois/SPaRKy/infiles/tplans/italianwestvillage/TomOstrand_k-0.7_recommend.xml");
    }

    public void test() {
        for (int i = 0; i < 20; i++) {
            testByPlan("/home/francois/SPaRKy/infiles/tplans/italianwestvillage/TomOstrand_k-0.7_recommend.xml");
        }
    }

    public void test2() {
        List lis = generateAlt(
                "/home/francois/SPaRKy/infiles/tplans/upperwestsideasian/RachelOstrand_k-0.7_recommend.xml",
                20);
        System.out.println("generated " + lis.size() + " sentences");
        for (int i = 0; i < lis.size(); i++) {
            System.out.println("[" + i + "] " + lis.get(i));
        }
    }

    public void test3() {
        List lis = getTplans("recommend");
        for (int i = 0; i < lis.size(); i++) {
            String tplan = (String) lis.get(i);
            List alt = generateAlt(tplan, 20);
            System.out.println("[alt] generated " + alt.size()
                    + " sentences for " + tplan);
            for (int j = 0; j < alt.size(); j++) {
                System.out.println("[alt" + j + "] " + alt.get(j));
            }
        }
    }
    


    /** main * */
    public static void main(String args[]) {
        App a = new App("/home/francois/SPaRKy/infiles/dss");
        //a.testAllCategory();

        a.test2();

        //a.test3();
    }
}
