/** Proposition * */
package jsparky;

import java.util.*;
import org.w3c.dom.*;

public class Proposition {

    private String id;

    private Document dss;

    private String sentence;

    private String dssname;

    private Element prop_elem;

    public Proposition(Element prop_elem) {
        this.id = prop_elem.getAttribute("id");
        this.dssname = prop_elem.getAttribute("dialogue_act");
        this.prop_elem = prop_elem;
    }

    public String getId() {
        return id;
    }

    public String getDSSName() {
        return dssname;
    }

    public Document getDSS() {
        return dss;
    }

    public String toString() {
        return "Proposition id=" + id;
    }

    public String getAttribute(String key) {
        List lis = Utils.nodeListToList(prop_elem
                .getElementsByTagName("attribute"));
        for (int i = 0; i < lis.size(); i++) {
            Element el = (Element) lis.get(i);
            if (el.getAttribute("name").equals(key)) {
                return el.getAttribute("value");
            }
        }
        return "";
    }

    public void setDSS(Document dss) {
        this.dss = dss;
    }

    public Document fillDSS(Document doc) {
        fillDSS(doc.getDocumentElement());
        return doc;
    }

    public void fillDSS() {
        if (dss != null) {
            fillDSS(dss.getDocumentElement());
        }
    }

    /** Fill the arguments of DSyntS based on proposition attributes, modifying the node's lexeme. * */
    protected void fillDSS(Element parent) {
        for (Node child = parent.getFirstChild(); child != null; child = child
                .getNextSibling()) {
            if (child.getNodeType() == Node.ELEMENT_NODE) {
                Element elem = (Element) child;
                String lexeme = elem.getAttribute("lexeme");
                // if DSyntS node contains variable
                if (lexeme.indexOf("_X") != -1) {
                    String arg = null;
                    String tokens[] = lexeme.split("_");
                    arg = tokens[1];
                    NodeList nl = prop_elem.getElementsByTagName("attribute");
                    // for each attribute of the proposition
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element attr = (Element) nl.item(i);
                        // check if entity name
                        if (attr.getAttribute("name").equals("SELECTION")) {
                            elem.setAttribute("restaurant_name", "true");
                        }
                        // check if arguments match
                        if (attr.getAttribute("arg").equals(arg)) {
                            String value = attr.getAttribute("value");
                            String values[] = value.split(",");
                            // realize lexeme for multiple values (inserting conjunction)
                            if (values.length == 2) {
                                value = values[0] + " and " + values[1];
                                elem.setAttribute("number", "pl");
                            } else if (values.length == 3) {
                                value = values[0] + ", " + values[1] + " and "
                                        + values[2];
                                elem.setAttribute("number", "pl");
                            }
                            // add genitive if lexeme ends with "_s"? TODO better
                            if ((!value.endsWith("'s")) && tokens.length == 3) {
                                value += "'s";
                            }
                            if (elem.getAttribute("class").equals("adjective")) {
                                value = value.toLowerCase();
                            }
                            value = value.replaceAll("\\s", "_");
                            elem.setAttribute("lexeme", value);
                            break;
                        }
                    }
                }
                fillDSS(elem);
            }
        }
    }
}
