/** Relation * */
package jsparky;

import java.util.*;

public class Relation {

    private String relName;

    private ArrayList propositions;

    private Hashtable nsTable;

    /** global constants * */
    public static final String NUCLEUS = "nucleus";

    public static final String SATELLITE = "satellite";

    public static final String JUSTIFY = "justify";

    public static final String CONTRAST = "contrast";

    public static final String INFER = "infer";

    public static final String ELABORATION = "elaboration";

    public static final String[] RELATION_NAMES = { "justify", "contrast",
            "infer", "elaboration" };

    public static boolean isMonoNuclear(String rel_name) {
        if (rel_name.equals(Relation.JUSTIFY)) {
            return true;
        } else if (rel_name.equals(Relation.CONTRAST)) {
            return false;
        } else if (rel_name.equals(Relation.INFER)) {
            return false;
        } else if (rel_name.equals(Relation.ELABORATION)) {
            return true;
        } else {
            return false;
        }
    }

    public Relation(String rel_name) {
        this.relName = rel_name;
        propositions = new ArrayList();
        nsTable = new Hashtable();
        //System.err.println("[Relation] " + rel_name + " created");
    }

    public void addProposition(Proposition prop, String ns) {
        propositions.add(prop);
        nsTable.put(prop, ns);
    }

    public String getRelationName() {
        return relName;
    }

    public String getNS(Proposition prop) {
        return (String) nsTable.get(prop);
    }

    public ArrayList getPropositions() {
        return propositions;
    }

    public ArrayList getNucleus() {
        return getPropositions(Relation.NUCLEUS);
    }

    public ArrayList getSatellite() {
        return getPropositions(Relation.SATELLITE);
    }

    public ArrayList getPropositions(String ns) {
        ArrayList lis = new ArrayList();
        for (int i = 0; i < propositions.size(); i++) {
            Proposition prop = (Proposition) propositions.get(i);
            String s = (String) nsTable.get(prop);
            if (s.equals(ns)) {
                lis.add(prop);
            }
        }
        return lis;
    }

    public ArrayList getNSOrder() {
        ArrayList lis = new ArrayList();
        for (int i = 0; i < propositions.size(); i++) {
            Proposition prop = (Proposition) propositions.get(i);
            String s = (String) nsTable.get(prop);
            lis.add(s);
        }
        return lis;
    }

}
