/** 
    CueWord Because SN Operation
    first created by Ryuichiro higashinaka 
    2005/06/01
**/

package jsparky.operations;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;
import com.cogentex.real.api.*;

public class CWBecauseSN extends Operation {

    public boolean isInvalidOperationInStructure(String op_name){
	if(op_name.startsWith("PERIOD")){
	    return true;
	}
	return false;
    }

    public AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException {

	/** check structure **/
	checkInvalidOperationInStructure(a1,a2);

	String relation = rst_elem.getRelationName();
	if(! relation.equals(Relation.JUSTIFY)){
	    throw new AggregationException("not justify",this);
	}

	if(! ( a1.getNS().equals(Relation.SATELLITE) && a2.getNS().equals(Relation.NUCLEUS))){
	    throw new AggregationException("invalid ns order",this);
	}
	
	Document dss1 = a1.getDSS();
	Document dss2 = a2.getDSS();
	Element root1 = dss1.getDocumentElement();
	Element root2 = dss2.getDocumentElement();
	Element pred1 = (Element)root1.getFirstChild();
	Element pred2 = (Element)root2.getFirstChild();

	// no a verb (2005/06/13)
	if(! pred1.getAttribute("class").equals("verb") || ! pred2.getAttribute("class").equals("verb")){
	    throw new AggregationException("not a verb",this);
	}
	
	List and1 = findAND(pred1);
	if(and1.size() > 0){
	    throw new AggregationException("and exists",this);
	}
	
	Element because = dss1.createElement(RealProMgr.TAG_DSYNTNODE);
	because.setAttribute("lexeme","because");
	because.setAttribute("rel","ATTR");
	because.setAttribute("class","adverb");
	because.setAttribute("starting_point","+");
	pred1.setAttribute("rel","II");
	because.appendChild(pred1);
	Element scope = dss1.createElement("rightmost_comma");
	scope.appendChild(because);
	//pred2.appendChild(dss2.importNode(because,true));
	pred2.appendChild(dss2.importNode(scope,true));

	Document newdss = new DocumentImpl();
	newdss.appendChild(newdss.importNode(root2,true));
	return new AggregationResult(newdss,dss1,dss2);
    }

    // find specific "and" 
    public static List findAND(Element elem){
	ArrayList lis = new ArrayList();
	NodeList nl = elem.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element el = (Element)nl.item(i);
	    if(el.getAttribute("lexeme").equals("AND") 
	       && el.getAttribute("rel").equals("COORD") && el.getAttribute("punct").equals("replace_by_comma")){
		lis.add(el);
	    }
	}
	return lis;
    }

    /** each operation has to return its name **/
    public String getOperationName(){
	return Operation.CW_BECAUSE_SN;
    }

}
