/** 
    Relative Clause Operation 
    first created by Ryuichiro higashinaka 
    2005/05/31
**/

package jsparky.operations;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;
import com.cogentex.real.api.*;

public class RelativeClause extends Operation {

    /** embedding check **/
    public boolean isInvalidOperationInStructure(String op_name){
	if(op_name.startsWith("PERIOD") || op_name.startsWith("CW") || op_name.startsWith("RELATIVE_CLAUSE")){
	    return true;
	}
	return false;
    }

    public AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException {

	/** check structure **/
	checkInvalidOperationInStructure(a1,a2);

	String relation = rst_elem.getRelationName();
	if(! (relation.equals(Relation.INFER) || relation.equals(Relation.JUSTIFY))){
	    throw new AggregationException("Not infer or justify",this);
	}
	Document dss1 = a1.getDSS();
	Document dss2 = a2.getDSS();
	Element root1 = dss1.getDocumentElement();
	Element root2 = dss2.getDocumentElement();
	Element pred1 = (Element)root1.getFirstChild();
	Element pred2 = (Element)root2.getFirstChild();
	
	if(! pred1.getAttribute("lexeme").equals("BE3") && ! pred2.getAttribute("lexeme").equals("BE3")){
	    throw new AggregationException("one element should have BE3",this);
	}
	List lis = Utils.getElementsByTagAndAttr(pred1,RealProMgr.TAG_DSYNTNODE,"rel","COORD");
	if(lis.size()>0){
	    throw new AggregationException("coordination found in a1",this);
	}
	Element pred1relI = Utils.getFirstElementByTagAndAttr(pred1,RealProMgr.TAG_DSYNTNODE,"rel","I");
	Element pred2relI = Utils.getFirstElementByTagAndAttr(pred2,RealProMgr.TAG_DSYNTNODE,"rel","I");
	if(! pred1relI.getAttribute("lexeme").equals(pred2relI.getAttribute("lexeme"))){
	    throw new AggregationException("different relI constituent in a1 and a2",this);	    
	}
	Document newdss = new DocumentImpl();
	pred1relI.setAttribute("ref","r1");
	pred2relI.setAttribute("ref","r1");
	if(pred1.getAttribute("lexeme").equals("BE3")){
	    pred2.setAttribute("rel","DESC-ATTR");
	    pred1relI.appendChild(dss1.importNode(pred2,true));
	    newdss.appendChild(newdss.importNode(root1,true));
	}else{
	    pred1.setAttribute("rel","DESC-ATTR");
	    pred2relI.appendChild(dss2.importNode(pred1,true));
	    newdss.appendChild(newdss.importNode(root2,true));
	}
	//return newdss;
	return new AggregationResult(newdss,dss1,dss2);
    }

    /** each operation has to return its name **/
    public String getOperationName(){
	return Operation.RELATIVE_CLAUSE;
    }

}
