/** alt generation program **/

package jsparky.review;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import jsparky.operations.*;
import jsparky.*;
import org.apache.commons.cli.*;

public class AltGenerator extends Object {

    // alternative sentences' num
    public int alt_num = 10;

    /** generate sentences **/    
    public void generate(App a1, AppEx a2, String outdir){
	File file = new File(outdir);
	file.mkdirs();
	String categories[] = {"cheap","french","eastvillagejapanese","italianwestvillage","midtownwest","upperwestsideasian"};
	for(int i=0;i<categories.length;i++){
	    File category_dir = createDir(outdir + File.separator + categories[i]);
	    List tplans = a1.getTplans("recommend",categories[i]);
	    for(int j=0;j<tplans.size();j++){
		String tplan = (String)tplans.get(j);
		File tplan_file = new File(tplan);
		String tplan_name = tplan_file.getName();
		tplan_name = tplan_name.substring(0,tplan_name.lastIndexOf(".xml"));
		System.err.println("tplan_name : " + tplan_name);
		File tplan_dir = createDir(category_dir + File.separator + tplan_name);
		File sparky_dir = createDir(tplan_dir + File.separator + "sparky");
		File sparky_review_dir = createDir(tplan_dir + File.separator + "sparky_review");
		// write output for sparky
		List sparky_alt = a1.generateAlt(tplan,alt_num);
		write_results_to_dir(a1,sparky_alt,sparky_dir);
		// write output for sparky with review
		List sparky_review_alt = a2.generateAlt(tplan,alt_num);
		write_results_to_dir(a2,sparky_review_alt,sparky_review_dir);
		// write about content
		SentencePlan first_splan = (SentencePlan)sparky_alt.get(0);
		String content_str = getContentString(first_splan);
		OutputToFile(tplan_dir.getAbsolutePath() + File.separator + "content.txt", content_str);
	    }
	}
    }

    // create dir utility
    public File createDir(String dirname){
	File dir = new File(dirname);
	if(dir.mkdirs()){
	    System.err.println(dirname + " created");
	}else{
	    System.err.println("ERROR: " + dirname + " not created");
	}
	return dir;
    }

    // write alternative information
    public void write_results_to_dir(App a, List lis, File dir){
	// write sents to the top dir
	StringWriter sw = new StringWriter();
	sw.write("# generated " + lis.size() + " sentences.\n");
	for(int i=0;i<lis.size();i++){
	    SentencePlan splan = (SentencePlan)lis.get(i);
	    String final_text = a.getFinalText(splan);
	    int altnum = i+1;
	    sw.write("[" + altnum + "] " + final_text + "\n");
	}
	OutputToFile(dir.getAbsolutePath() + File.separator + "sents.txt", sw.toString());
	// for each alternative 
	for(int i=0;i<lis.size();i++){
	    SentencePlan splan = (SentencePlan)lis.get(i);
	    RSTPlan rstplan = splan.getRSTPlan();
	    String final_text = a.getFinalText(splan);
	    String serialized_splan = Utils.serialize(splan);
	    String serialized_rstplan = Utils.serialize(rstplan);
	    List dssList = Utils.getAllDSS(splan);
	    int altnum = i + 1;
	    File altdir = createDir(dir.getAbsolutePath() + File.separator + "alt" + altnum);
	    String altdir_path = altdir.getAbsolutePath();
	    // output
	    OutputToFile(altdir_path + File.separator + "sent.txt", final_text);
	    OutputToFile(altdir_path + File.separator + "splan.xml", serialized_splan);
	    OutputToFile(altdir_path + File.separator + "rstplan.xml", serialized_rstplan);
	    for(int j=0;j<dssList.size();j++){
		Document dss = (Document)dssList.get(j);
		int dssnum = j + 1;
		String serialized_dss = Utils.serialize(dss);
		String dss_string = Realizer.realproManager.toDSSString(dss.getDocumentElement(),true,"");
		OutputToFile(altdir_path + File.separator + "dss" + dssnum + ".xml", serialized_dss);
		OutputToFile(altdir_path + File.separator + "dss" + dssnum + ".dss", dss_string);
	    }
	}
    }

    /** pretty print the propositions **/
    public String getContentString(SentencePlan splan){
	List props = Utils.getPropositionList((AggregatedElement)splan.getDocumentElement());
	Content content = new Content();
	int price = 0;
	for(int i=0;i<props.size();i++){
	    Proposition prop = (Proposition)props.get(i);
	    String attr = prop.getAttribute("ATTR");
	    if(attr.equals("BEST")){
		// do nothing here
	    }
	    else if(attr.equals("PRICE")){
		try{
		    price = Integer.parseInt(prop.getAttribute("MOD_REAL"));
		}catch(NumberFormatException e){
		    e.printStackTrace();
		}
	    }
	    else{
		Content c = new Content(prop);
		try{
		    content.addContent(c);
		}catch(Exception e){
		    e.printStackTrace();
		}
	    }
	}
	StringWriter sw = new StringWriter();
	Hashtable info = content.getInfo();
	List keys = new ArrayList(info.keySet());
	Collections.sort(keys);
	for(int i=0;i<keys.size();i++){
	    String name = (String)keys.get(i);
	    String value = content.getInfo(name);
	    sw.write(name + "=" + value + "\n");
	}
	Hashtable scores = content.getScores();
	keys = new ArrayList(scores.keySet());
	Collections.sort(keys);
	for(int i=0;i<keys.size();i++){
	    String name = (String)keys.get(i);
	    int score = content.getScore(name);
	    sw.write(name + "=" + score + "\n");
	}
	if(price > 0){
	    sw.write("price=" + price + "\n");
	}
	return sw.toString();
    }

    public void setAltNum(int alt_num){
	this.alt_num = alt_num;
    }

    /** simple file output **/
    public void OutputToFile(String filename, String content){
	try{
	    BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename)));
	    bw.write(content);
	    bw.flush();
	    bw.close();
	}catch(Exception e){
	    e.printStackTrace();
	}
    }

    /** main **/
    public static void main(String args[]){
	App a = new App("./dss");
	AppEx b = new AppEx("./dss","./dssconverter/test_outdir");
	AltGenerator generator = new AltGenerator();
	if(args.length == 1){
	    try{
		generator.setAltNum(Integer.parseInt(args[0]));
	    }catch(NumberFormatException e){
		e.printStackTrace();
	    }
	}
	generator.generate(a,b,"alternatives");
    }
}

