/** review based application class **/

package jsparky.review;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import jsparky.operations.*;
import jsparky.*;

public class AppEx extends jsparky.App {

    ReviewManager review_manager;
    TermSwap swap;
    public static final String SWAP_DIC = "C:\\home\\Project\\Java\\ReviewCollector\\perl\\adj";

    public AppEx(String sparky_dss_dir, String review_dss_dir){
	super(sparky_dss_dir);
	review_manager.setRealizer(realizer);
	review_manager.loadDSS(review_dss_dir);
	swap = new TermSwap(SWAP_DIC);
    }

    public OperationManager createOperationManager(){
	review_manager = new ReviewManager();
	return new OperationManagerEx(review_manager,false);//random replace
	//return new OperationManagerEx(review_manager,true);
    }

    /** four stage process **/
    public SentencePlan _process(String speech_plan_path){
	SpeechPlan splan = new SpeechPlan(speech_plan_path);
	splan.bindDSS(realizer);
	try{
	    RSTPlan rstplan = transformer.transform(splan.getRelations());
	    swap.swapAdjective(rstplan);
	    System.out.println(Utils.serialize(rstplan.getDocumentElement()));
	    SentencePlan sentplan = manager.aggregate(rstplan);
	    showResult(sentplan);
	    return sentplan;
	}catch(Exception e){
	    e.printStackTrace();
	}
	return null;
    }

    // for AppEx
    public void showResult(SentencePlan splan){
	AggregatedElement a = (AggregatedElement)splan.getDocumentElement();
	String text = manager.getText(a);
	String newtext = Utils.process_rightmost_comma(text);
	if(review_manager.hasReviewElement(a)){
	    System.out.println("\nfinal : " + newtext + "**\n");
	}else{
	    System.out.println("\nfinal : " + newtext + "\n");
	}
    }
    
    public boolean hasReviewElement(SentencePlan splan){
	AggregatedElement a = (AggregatedElement)splan.getDocumentElement();	
	if(review_manager.hasReviewElement(a)){
	    return true;
	}
	return false;
    }

    // generate different num sentences of tplan 
    public List generateAlt(String tplan, int num){
	ArrayList lis = new ArrayList();
	ArrayList splan_lis = new ArrayList();
	int max = 1000;
	for(int i=0;i<max;i++){
	    SentencePlan splan = process(tplan);
	    if(splan != null && hasReviewElement(splan)){
		String final_text = getFinalText(splan);
		if(! lis.contains(final_text)){
		    lis.add(final_text);
		    splan_lis.add(splan);
		}
	    }
	    if(lis.size()>=num){
		return splan_lis;
	    }
	}
	System.out.println("Warning : " + num + " not reached within " + max + " attempts");
	return splan_lis;
    }

    public void test(){
	for(int i=0;i<20;i++){
	    testByPlan("./tplans/upperwestsideasian/RachelOstrand_k-0.7_recommend.xml");
	}
    }

    public void test2(){
	//List lis = generateAlt("./tplans/upperwestsideasian/RachelOstrand_k-0.7_recommend.xml",20);
	manager.setDebug(true);
	List lis = generateAlt("./tplans/eastvillagejapanese/OwenRambow_k-0.7_recommend.xml",20);
	System.out.println("generated " + lis.size() + " sentences");
	for(int i=0;i<lis.size();i++){
	    String final_text = getFinalText((SentencePlan)lis.get(i));
	    System.out.println("[" + i + "] " + final_text);
	}
    }

    public void test4(){
	List lis = generateAlt("./tplans/upperwestsideasian/TomOstrand_k-0.7_recommend.xml",20);
	System.out.println("generated " + lis.size() + " sentences");
	for(int i=0;i<lis.size();i++){
	    System.out.println("[" + i + "] " + lis.get(i));
	}
    }

    public void test3(){
	List lis = getTplans("recommend");
	for(int i=0;i<lis.size();i++){
	    String tplan = (String)lis.get(i);
	    List alt = generateAlt(tplan,20);
	    System.out.println("[alt] generated " + alt.size() + " sentences for " + tplan);
	    for(int j=0;j<alt.size();j++){
		System.out.println("[alt" + j + "] " + alt.get(j));
	    }
	}
    }

    /** main **/
    public static void main(String args[]){
	//AppEx a = new AppEx("./dss","./review_dss");
	AppEx a = new AppEx("./dss","./dssconverter/test_outdir");
	//a.testAllCategory();
	//a.testRecommend();
	//a.testSimple();
	//a.test();
	a.test2();
	//a.test4();
    }
}
