/** User Review Content **/

package jsparky.review;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import com.cogentex.real.api.*;
import java.io.*;

public class Content extends Object {

    /** globals **/
    Hashtable info;
    Hashtable scores;
    String desc = "";

    /** user review field constants **/
    public static final String FOOD = "food";
    public static final String SERVICE = "service";
    public static final String VALUE = "value";
    public static final String ATMOSPHERE = "atmosphere";
    public static final String OVERALL = "overall";

    public static final String RESTAURANT_NAME = "restaurantname";
    public static final String PLACE_NAME = "placename"; // same as Location
    public static final String LOCATION = "location";    
    public static final String FOOD_TYPE = "foodtype";

    public static final String DELIMITER = "_";
    public static final String SCORE_DELIMITER = "-";

    /** null constructor **/
    public Content(){
	info = new Hashtable();
	scores = new Hashtable();
    }

    /** abstracting proposition as a content **/
    public Content(Proposition prop){
	this();
	String attr = prop.getAttribute("ATTR");
	attr = attr.replaceAll("\\s","_");
	info.put(RESTAURANT_NAME,prop.getAttribute("SELECTION"));
	int score = 0;
	try{score = translateScore(Integer.parseInt(prop.getAttribute("MOD_REAL")));}
	catch(NumberFormatException e){}
	if(attr.equals("FOOD_QUALITY")){
	    setScore(FOOD,score);
	    desc = FOOD + SCORE_DELIMITER + score;
	}
	else if(attr.equals("SERVICE")){
	    setScore(SERVICE,score);
	    desc = SERVICE + SCORE_DELIMITER + score;
	}
	else if(attr.equals("DECOR")){
	    setScore(ATMOSPHERE,score);
	    desc = ATMOSPHERE + SCORE_DELIMITER + score;
	}
	else if(attr.equals("NBHD")){
	    info.put(LOCATION,prop.getAttribute("MOD_LEX"));
	    desc = LOCATION;
	}
	else if(attr.equals("CUISINE")){
	    info.put(FOOD_TYPE,prop.getAttribute("MOD_LEX"));
	    desc = FOOD_TYPE;
	}
	/**
	else if(attr.equals("LIST_EXCEPTIONAL")){
	    setScore(VALUE,5);
	    desc = VALUE + SCORE_DELIMITER + 5;
	}
	**/
	/**
	else if(attr.equals("BEST")){
	    setScore(OVERALL,5);
	    desc = OVERALL + SCORE_DELIMITER + 5;
	}
	**/
    }

    /** fill content by a description **/
    public Content(String desc) throws ContentException{
	this();
	desc = removeId(desc);
	String tokens[] = desc.split(DELIMITER);
	for(int i=0;i<tokens.length;i++){
	    String token = tokens[i];
	    if(token.equals(RESTAURANT_NAME) || token.equals(LOCATION) || token.equals(PLACE_NAME) || token.equals(FOOD_TYPE)){
		// ad hoc here
		if(token.equals(PLACE_NAME)){
		    token = LOCATION;
		}
		if(info.get(token)!=null){
		    throw new ContentException("multiple slot fields found");
		}
		info.put(token,token);
	    }else{
		String lis[] = token.split(SCORE_DELIMITER);
		scores.put(lis[0],new Integer(Integer.parseInt(lis[1])));
	    }
	}
	this.desc = desc;
	System.out.println("desc => " + this);
    }

    public String removeId(String desc){
	int pos = desc.lastIndexOf("_");
	String id = desc.substring(pos+1);
	//System.out.println("processing : id=" + id);
	try{
	    int id_num = Integer.parseInt(id);
	    desc = desc.substring(0,pos);
	}catch(NumberFormatException e){}
	return desc;
    }

    /** change scale to 1-5 **/
    /**
    protected int translateScore(int real){
	if(20 <= real){
	    return 5;
	}
	else if(17 <= real){
	    return 4;
	}
	else if(14 <= real){
	    return 3;
	}
	else if(11 <= real){
	    return 2;
	}
	else{
	    return 1;
	}
    }
    **/

    /** old version **/
    protected int translateScore(int real){
	if(23 <= real){
	    return 5;
	}
	else if(20 <= real){
	    return 4;
	}
	else if(17 <= real){
	    return 3;
	}
	else if(14 <= real){
	    return 2;
	}
	else{
	    return 1;
	}
    }

    /** add content **/
    public void addContent(Content c) throws ContentException {
	System.out.println("[addContent] " + this + " + " + c);
	Utils.addHashtable(info, c.getInfo());	
	Utils.addHashtable(scores, c.getScores());
	// for price and so on. (2005/06/13)
	if(c.getInfo(RESTAURANT_NAME) != null && c.getInfo().size() == 1 && c.getScores().size() == 0){
	    throw new ContentException("adding null content");
	}
	if(desc.equals("")){
	    setDesc(c.getDesc());
	}else{
	    setDesc(desc + DELIMITER + c.getDesc());
	}
    }

    /** scores **/    
    public Hashtable getScores(){
	return scores;
    }

    public void setScore(String key, int score){
	scores.put(key,new Integer(score));
    }

    public int getScore(String key){
	Integer integer = (Integer) scores.get(key);
	if(integer != null){
	    return integer.intValue();
	}
	return 0;
    }

    /** other information **/    
    public Hashtable getInfo(){
	return info;
    }

    public String getInfo(String name){
	Object obj = info.get(name);
	if(obj instanceof String){
	    return (String)obj;
	}
	return null;
    }

    /** descripter; name **/
    public String getDesc(){
	return desc;
    }

    public void setDesc(String desc){
	this.desc = desc;
    }
    
    /** toString **/
    public String toString(){
	return info + " " + scores + " desc=" + desc;
    }
    
}

